package ch.std.meteo.test.service;

import static org.junit.jupiter.api.Assertions.assertArrayEquals;
import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertNotNull;
import static org.mockito.Mockito.when;

import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.orm.jpa.DataJpaTest;
import org.springframework.boot.test.mock.mockito.MockBean;
import org.springframework.context.annotation.ComponentScan;
import org.springframework.test.context.ActiveProfiles;
import org.springframework.test.context.junit.jupiter.SpringExtension;

import ch.std.meteo.jpa.MeteoData;
import ch.std.meteo.repositories.MeteoDataRepository;
import ch.std.meteo.service.MeteoService;
import ch.std.meteo.service.MeteoService.MeteoMetrics;
import ch.std.meteo.test.setup.MeteoDataTestSetup;

@ExtendWith(SpringExtension.class)
@DataJpaTest
@ComponentScan(basePackages = {"ch.std.meteo.test"}, basePackageClasses = {MeteoService.class})
@ActiveProfiles("unittest")
public class MeteoServiceTestUsingMockBean {
	
	@MockBean
	private MeteoDataRepository meteoDataRepository;

	@Autowired
	private MeteoService meteoService;
	
	@BeforeEach
	public void setup() {
		when(meteoDataRepository.findAll()).thenReturn(Arrays.stream(MeteoDataTestSetup.TEMPERATURES).map(t -> new MeteoData(t)).collect(Collectors.toList()));
	}

	@Test
	public void test_getMeteoDaten() {
		List<MeteoData> meteoDataList = this.meteoService.getMeteoDaten();
		assertNotNull(meteoDataList);
		Double[] loadedTemperatures = meteoDataList.stream().map(md -> md.getTemperature()).toArray(Double[]::new);
		assertArrayEquals(MeteoDataTestSetup.TEMPERATURES, loadedTemperatures);		
	}
	
	@Test
	public void test_getMeteoMetrics() {
		MeteoMetrics meteoMetrics = this.meteoService.getMeteoMetrics();
		assertNotNull(meteoMetrics);
		assertEquals(MeteoDataTestSetup.MIN, meteoMetrics.getMin());
		assertEquals(MeteoDataTestSetup.MAX, meteoMetrics.getMax());	
		assertEquals(MeteoDataTestSetup.MED, meteoMetrics.getMed());	
	}
	
}
